/*
 * Copyright 1999, Alexander Feldman <alex@varna.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of Alexander Feldman nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ALEXANDER FELDMAN AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL ALEXANDER FELDMAN OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __IDEA_HPP__
#define __IDEA_HPP__

#include <string.h>

#include "globals.hpp"
#include "block.hpp"

class SCL_API CIDEAKey : public CBlockKey
{
	public:
		enum { KEYSIZE = 16 };

	public:
		CIDEAKey();
		CIDEAKey(const CIDEAKey &);
		CIDEAKey(const Byte *, Word = KEYSIZE);
		CIDEAKey(Dword, Dword);
		virtual ~CIDEAKey();

	public:
		Dword GetMaster(Word i) const { return dwMaster[i]; };
		Short GetKey(Word i) const { return sKey[i]; };

	public:
		virtual Word GetKeySize() { return KEYSIZE; };

	private:
		void MakeKeys();
		Short ModInv(Word, Word);

	private:
		Dword dwMaster[2];
		Short sKey[104];
};

class SCL_API CIDEABlock : public CBlockObject
{
	public:
		enum { BLOCKSIZE = 8 };
	
	public:
		CIDEABlock();
		CIDEABlock(const CIDEABlock &);
		CIDEABlock(Dword);
		CIDEABlock(const Byte *, Word = BLOCKSIZE);
		virtual ~CIDEABlock();

	public:
		virtual void Encrypt(const CBlockKey &cKey) { Encrypt((const CIDEAKey &)cKey); };
		virtual void Decrypt(const CBlockKey &cKey) { Decrypt((const CIDEAKey &)cKey); };
		virtual void SetData(const Byte *, Word = BLOCKSIZE);
		virtual Byte *GetData();
		virtual Word GetBlockSize() { return BLOCKSIZE; };

	public:
		void Encrypt(const CIDEAKey &);
		void Decrypt(const CIDEAKey &);
		void SetValue(Dword);
		Dword GetValue();

	private:
		void Crypt(const CIDEAKey &, bool);
		Short Mul(Short, Short);

	private:
		Dword dwData, dwResult;
};

#endif // __IDEA_HPP__
