/*
 * Copyright 1999, Alexander Feldman <alex@varna.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of Alexander Feldman nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ALEXANDER FELDMAN AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL ALEXANDER FELDMAN OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __LOKI_HPP__
#define __LOKI_HPP__

#include <string.h>

#include "globals.hpp"
#include "block.hpp"

class SCL_API CLOKIKey : public CBlockKey
{
	public:
		enum { KEYSIZE = 16, MAXKEYSZIE = 32 };

	public:
		CLOKIKey();
		CLOKIKey(const CLOKIKey &);
		CLOKIKey(const Byte *, Word);
		CLOKIKey(Dword, Dword, Dword, Dword);
		CLOKIKey(Dword, Dword, Dword);
		CLOKIKey(Dword, Dword);
		virtual ~CLOKIKey();

	public:
		Dword GetMaster(int i) const { return dwMaster[i]; };
		Dword GetKey(int i) const { return dwKey[i]; };
	
	public:
		virtual Word GetKeySize() { return wKeyLength; };

	private:
		void MakeKeys();

	private:
		Word wKeyLength;
		Dword dwMaster[4];
		Dword dwKey[48];
};

class SCL_API CLOKIBlock : public CBlockObject
{
	public:
		enum { BLOCKSIZE = 16 };

	public:
		CLOKIBlock();
		CLOKIBlock(const CLOKIBlock &);
		CLOKIBlock(Dword, Dword);
		CLOKIBlock(const Byte *, Word = BLOCKSIZE);
		virtual ~CLOKIBlock();

 	public:
		virtual void Encrypt(const CBlockKey &cKey) { Encrypt((const CLOKIKey &)cKey); };
		virtual void Decrypt(const CBlockKey &cKey) { Decrypt((const CLOKIKey &)cKey); };
		virtual void SetData(const Byte *, Word = BLOCKSIZE);
		virtual Byte *GetData();
		virtual Word GetBlockSize() { return BLOCKSIZE; };

	public:
		void Encrypt(const CLOKIKey &);
		void Decrypt(const CLOKIKey &);
		void SetData(Dword, Dword);
		Dword GetData(Word);
	
	private:
		Dword dwData[2];
		Dword dwResult[2];
};

#endif // __LOKI_HPP__
