/*
 * Copyright 1999, Alexander Feldman <alex@varna.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of Alexander Feldman nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ALEXANDER FELDMAN AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL ALEXANDER FELDMAN OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __MARS_HPP__
#define __MARS_HPP__

#include <string.h>

#include "globals.hpp"
#include "block.hpp"

class SCL_API CMARSKey : public CBlockKey
{
	public:
		enum { KEYSIZE = 16, MAXKEYSIZE = 56 };

	public:
		CMARSKey();
		CMARSKey(const CMARSKey &);
		CMARSKey(const Word *, Word);
		CMARSKey(const Byte *, Word = KEYSIZE);
		CMARSKey(Dword, Dword);
		virtual ~CMARSKey();

	public:
		Word GetMaster(Word i) const { return wMaster[i]; };
		Word GetKey(Word i) const { return wKey[i]; };
		const Word *GetKeys() const { return wKey; };

	public:
		virtual Word GetKeySize() { return wKeyLength * BYTESINWORD; };

	private:
		void MakeKeys();

	private:
		Word wMaster[14];						// Minimum 128 bits (4 words)
		Word wKey[40];
		Word wKeyLength;
};

class SCL_API CMARSBlock : public CBlockObject
{
	public:
		enum { BLOCKSIZE = 16 };

	public:
		CMARSBlock();
		CMARSBlock(const CMARSBlock &);
		CMARSBlock(Dword, Dword);
		CMARSBlock(const Byte *, Word = BLOCKSIZE);
		virtual ~CMARSBlock();

	public:
		virtual void Encrypt(const CBlockKey &cKey) { Encrypt((const CMARSKey &)cKey); };
		virtual void Decrypt(const CBlockKey &cKey) { Decrypt((const CMARSKey &)cKey); };
		virtual void SetData(const Byte *, Word = BLOCKSIZE);
		virtual Byte *GetData();
		virtual Word GetBlockSize() { return BLOCKSIZE; };

	public:
		void Encrypt(const CMARSKey &);
		void Decrypt(const CMARSKey &);
		void SetData(Word, Word, Word, Word);
		Word GetData(Word);

	private:
		Word wData[4];
};

#endif // __MARS_HPP__
