/*
 * Copyright 1999, Alexander Feldman <alex@varna.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of Alexander Feldman nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ALEXANDER FELDMAN AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL ALEXANDER FELDMAN OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __SHARK_HPP__
#define __SHARK_HPP__

#include <string.h>

#include "globals.hpp"
#include "block.hpp"

#define SHARKROUNDS 6

class SCL_API CSHARKKey : public CBlockKey
{
	public:
		enum { KEYSIZE = 16 };

	public:
		CSHARKKey();
		CSHARKKey(const CSHARKKey &);
		CSHARKKey(const Byte *, Word = KEYSIZE);
		virtual ~CSHARKKey();

	public:
		Byte GetMaster(Word i) const { return bMaster[i]; };
		const Dword *GetEncryptionKeys() const { return dwEncryptionKey; };
		const Dword *GetDecryptionKeys() const { return dwDecryptionKey; };
		Dword GetEncryptionKey(Word wKey) const { return dwEncryptionKey[wKey]; };
		Dword GetDecryptionKey(Word wKey) const { return dwDecryptionKey[wKey]; };

	public:
		virtual Word GetKeySize() { return KEYSIZE; };

	private:
		void MakeKeys();

	private:
		Byte bMaster[16];
		Dword dwEncryptionKey[SHARKROUNDS + 1];
		Dword dwDecryptionKey[SHARKROUNDS + 1];
};

class SCL_API CSHARKBlock : public CBlockObject
{
	public:
		enum { BLOCKSIZE = 8 };

	public:
		CSHARKBlock();
		CSHARKBlock(const CSHARKBlock &);
		CSHARKBlock(Dword);
		CSHARKBlock(const Byte *, Word = 8);
		virtual ~CSHARKBlock();

	public:
		virtual void Encrypt(const CBlockKey &cKey) { Encrypt((const CSHARKKey &)cKey); };
		virtual void Decrypt(const CBlockKey &cKey) { Decrypt((const CSHARKKey &)cKey); };
		virtual void SetData(const Byte *, Word = BLOCKSIZE);
		virtual Byte *GetData();
		virtual Word GetBlockSize() { return BLOCKSIZE; };

	public:
		void Encrypt(const CSHARKKey &);
		void Decrypt(const CSHARKKey &);
		void SetValue(Dword);
		Dword GetValue();

	private:
		Dword dwData, dwResult;
};

#endif // __SHARK_HPP__
